@@ -5,14 +5,16 @@ import datetime
 from sql import Union, Join, Select, Table, Null, As
 from sql.conditionals import Greatest
 
+from trytond.i18n import gettext
 from trytond.pool import PoolMeta, Pool
 from trytond.model import ModelView, Workflow, ModelSQL, ValueMixin, fields
+from trytond.model.exceptions import AccessError
 from trytond.pyson import Eval
 from trytond.transaction import Transaction
 from trytond.tools import grouped_slice
 
-__all__ = ['Configuration', 'ConfigurationLotShelfLife',
-    'Lot', 'Move', 'Period']
+from trytond.modules.stock.exceptions import PeriodCloseError
+from .exceptions import LotExpiredError, LotExpiredWarning
 
 DATE_STATE = [
     ('none', 'None'),
@@ -43,8 +45,7 @@ class ConfigurationLotShelfLife(ModelSQL, ValueMixin):
     shelf_life_delay = shelf_life_delay
 
 
-class Lot(metaclass=PoolMeta):
-    __name__ = 'stock.lot'
+class LotSledMixin:
 
     shelf_life_expiration_date = fields.Date('Shelf Life Expiration Date',
         states={
@@ -66,15 +67,6 @@ class Lot(metaclass=PoolMeta):
         fields.Selection(DATE_STATE, 'Expiration State'),
         'on_change_with_expiration_state')
 
-    @classmethod
-    def __setup__(cls):
-        super(Lot, cls).__setup__()
-        cls._error_messages.update({
-                'period_closed_expiration_dates': ('You can not modify '
-                    'the expiration dates of lot "%(lot)s" because '
-                    'it is used on a move "%(move)s" in a closed period'),
-                })
-
     @fields.depends('product')
     def on_change_with_shelf_life_expiration_state(self, name=None):
         if self.product:
@@ -92,7 +84,7 @@ class Lot(metaclass=PoolMeta):
         pool = Pool()
         Date = pool.get('ir.date')
         try:
-            super(Lot, self).on_change_product()
+            super().on_change_product()
         except AttributeError:
             pass
         if self.product:
@@ -106,6 +98,10 @@ class Lot(metaclass=PoolMeta):
                 self.expiration_date = (today
                     + datetime.timedelta(days=self.product.expiration_time))
 
+
+class Lot(LotSledMixin, metaclass=PoolMeta):
+    __name__ = 'stock.lot'
+
     @classmethod
     def write(cls, *args):
         super(Lot, cls).write(*args)
@@ -138,23 +134,32 @@ class Lot(metaclass=PoolMeta):
                         ]], limit=1)
             if moves:
                 move, = moves
-                cls.raise_user_error('period_closed_expiration_dates', {
-                        'lot': move.lot.rec_name,
-                        'move': move.rec_name,
-                        })
+                raise AccessError(
+                    gettext('stock_lot_sled'
+                        '.msg_lot_modify_expiration_date_period_close',
+                        lot=move.lot.rec_name,
+                        move=move.rec_name))
+
+
+class MoveAddLotsStartLot(LotSledMixin, metaclass=PoolMeta):
+    __name__ = 'stock.move.add.lots.start.lot'
+
+    @fields.depends('shelf_life_expiration_date', 'expiration_date')
+    def _set_lot_values(self, lot):
+        super()._set_lot_values(lot)
+        self.shelf_life_expiration_date = lot.shelf_life_expiration_date
+        self.expiration_date = lot.expiration_date
+
+    def _get_lot_values(self, move):
+        values = super()._get_lot_values(move)
+        values['shelf_life_expiration_date'] = self.shelf_life_expiration_date
+        values['expiration_date'] = self.expiration_date
+        return values
 
 
 class Move(metaclass=PoolMeta):
     __name__ = 'stock.move'
 
-    @classmethod
-    def __setup__(cls):
-        super(Move, cls).__setup__()
-        cls._error_messages.update({
-                'expiration_dates': ('The lot "%(lot)s" '
-                    'on move "%(move)s" is expired'),
-                })
-
     @classmethod
     @ModelView.button
     @Workflow.transition('done')
@@ -191,6 +196,7 @@ class Move(metaclass=PoolMeta):
         Group = pool.get('res.group')
         User = pool.get('res.user')
         ModelData = pool.get('ir.model.data')
+        Warning = pool.get('res.user.warning')
 
         types = cls.check_expiration_dates_types()
         locations = cls.check_expiration_dates_locations()
@@ -219,10 +225,15 @@ class Move(metaclass=PoolMeta):
                     'lot': move.lot.rec_name if move.lot else '',
                     }
                 if not in_group():
-                    cls.raise_user_error('expiration_dates', values)
+                    raise LotExpiredError(
+                        gettext('stock_lot_sled.msg_move_lot_expired',
+                            **values))
                 else:
-                    cls.raise_user_warning('%s.check_expiration_dates' % move,
-                        'expiration_dates', values)
+                    warning_name = '%s.check_expiration_dates' % move
+                    if Warning.check(warning_name):
+                        raise LotExpiredWarning(warning_name,
+                            gettext('stock_lot_sled.msg_move_lot_expired',
+                                **values))
 
     @classmethod
     def compute_quantities_query(cls, location_ids, with_childs=False,
@@ -332,10 +343,10 @@ class Move(metaclass=PoolMeta):
                         if add_join(sub_query.from_, lot):
                             # only lot expiring during the period
                             clause = (
-                                (lot.shelf_life_expiration_date >=
-                                    stock_date_start)
-                                & (lot.shelf_life_expiration_date <
-                                    stock_date_end))
+                                (lot.shelf_life_expiration_date
+                                    >= stock_date_start)
+                                & (lot.shelf_life_expiration_date
+                                    < stock_date_end))
                             if sub_query.where:
                                 sub_query.where &= clause
                             else:
@@ -352,15 +363,6 @@ class Move(metaclass=PoolMeta):
 class Period(metaclass=PoolMeta):
     __name__ = 'stock.period'
 
-    @classmethod
-    def __setup__(cls):
-        super(Period, cls).__setup__()
-        cls._error_messages.update({
-                'close_period_sled': ('You can not close a period '
-                    'before the Shelf Live Expiration Date "%(date)s" '
-                    'of Lot "%(lot)s"'),
-                })
-
     @classmethod
     @ModelView.button
     def close(cls, periods):
@@ -392,7 +394,7 @@ class Period(metaclass=PoolMeta):
             lot_id, = lot_id
             lot = Lot(lot_id)
             lang = Lang.get()
-            cls.raise_user_error('close_period_sled', {
-                    'date': lang.strftime(lot.shelf_life_expiration_date),
-                    'lot': lot.rec_name,
-                    })
+            raise PeriodCloseError(
+                gettext('stock_lot_sled.msg_period_close_sled',
+                    lot=lot.rec_name,
+                    date=lang.strftime(lot.shelf_life_expiration_date)))
